module adder_4_bit(output c_out, output [3:0] sum,
	input c_in, input [3:0] a, b);

  // uses 4 instances of the full_adder module
  wire [3:1] carry;
  full_adder fa0(.c_out(carry[1]), .sum(sum[0]),
	.c_in(c_in), .a(a[0]), .b(b[0]));
  full_adder fa1(.c_out(carry[2]), .sum(sum[1]),
  	.c_in(carry[1]), .a(a[1]), .b(b[1]));
  full_adder fa2(.c_out(carry[3]), .sum(sum[2]),
  	.c_in(carry[2]), .a(a[2]), .b(b[2]));
  full_adder fa3(.c_out(c_out), .sum(sum[3]),
  	.c_in(carry[3]), .a(a[3]), .b(b[3]));
endmodule

module adder_4_bit_gen(output c_out, output [3:0] sum,
	input c_in, input [3:0] a, b);

  // using generate to avoid repetition
  wire [4:0] carry;
  generate
    genvar i;
    for(i=0; i<4; i=i+1) begin: adder
      full_adder fa(.c_out(carry[i+1]), .sum(sum[i]),
	  	.c_in(carry[i]), .a(a[i]), .b(b[i]));
	end
  endgenerate
  
  assign carry[0] = c_in;
  assign c_out = carry[4];

endmodule
